package  
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.ui.Keyboard;
	import net.richardlord.coral.Matrix3d;
	import EqMath.*;
	/**
	 * ...
	 * @author Shiu
	 */
	[SWF(width = 500, height = 400)]
	public class CollisionTime2 extends Sprite
	{
		private var w1:Circle, w2:Circle, w3:Circle, collision1:Circle, collision2:Circle;
		private var ship:Triangle, velo:Vector2D;
		
		private var quadratic_equation:EqQuadratic;
		private var minX:Number;
		private var maxX:Number;
		private var n1:Matrix, n2:Matrix;
		private var m1:Matrix3d, m2:Matrix3d;
		
		private var control:int = 0, increase:int = 0;	
		
		//0 to maintain course, 1 to steer left, 2 to steer right
		//0 to decrease to initial speed, 1 to increase
		
		public function CollisionTime2() 
		{
			w1 = new Circle(0,5); addChild(w1); w1.name = "w1"
			w2 = new Circle(0, 5); addChild(w2); w2.name = "w2"
			w3 = new Circle(0,5); addChild(w3); w3.name = "w3"
			collision1 = new Circle(0xFF0000, 5); addChild(collision1);	collision1.name = "c0";
			collision2 = new Circle(0xFF0000, 5); addChild(collision2);	collision2.name = "c1";
			ship = new Triangle(); addChild(ship);
			
			w1.x = 20; w2.x = 250; 	w3.x = 450;	
			w1.y = 100;	w2.y = 150;	w3.y = 100;	
			
			collision1.x = -100;	collision2.x = -100;	ship.x = stage.stageWidth >> 1;
			collision1.y = -100;	collision2.y = -100;	ship.y = stage.stageHeight >> 1;
			
			velo = new Vector2D(1, -3);
			quadratic_equation = new EqQuadratic();
			
			stage.addEventListener(KeyboardEvent.KEY_DOWN, trigger);
			stage.addEventListener(KeyboardEvent.KEY_UP, trigger);
			stage.addEventListener(Event.ENTER_FRAME, update);
			stage.addEventListener(MouseEvent.MOUSE_DOWN, move);
			stage.addEventListener(MouseEvent.MOUSE_UP, stop);
		}
		
		private function move(e:MouseEvent):void 
		{
			if (e.target.name == "w1" || e.target.name == "w2" || e.target.name == "w3") e.target.startDrag();
		}
		
		private function stop(e:MouseEvent):void 
		{
			if (e.target.name == "w1" || e.target.name == "w2" || e.target.name == "w3") e.target.stopDrag();
		}
		
		private function trigger(e:KeyboardEvent):void 
		{
			if (e.type == "keyDown") {
				if (e.keyCode == Keyboard.LEFT) control = 1;
				else if (e.keyCode == Keyboard.RIGHT) control = 2;
				
				if (e.keyCode == Keyboard.UP) increase = 1;
			}
			if (e.type == "keyUp") { 
				if (e.keyCode == Keyboard.LEFT || e.keyCode == Keyboard.RIGHT) control = 0; 
				if (e.keyCode == Keyboard.UP || e.keyCode == Keyboard.DOWN)increase = 0;
			}
		}
		
		private function update(e:Event):void 
		{
			//Steering left and right
			if (control == 1)  velo = velo.rotate(Math2.radianOf(-5));
			else if (control == 2)  velo = velo.rotate(Math2.radianOf(5));
			
			//manipulating velocity
			var currVelo:Number = velo.getMagnitude();
			if (increase == 0)  {
				currVelo -= 0.5; currVelo = Math.max(currVelo, 1); //lower bound for velocity
			}
			else if (increase == 1)  {
				currVelo += 0.5; currVelo = Math.min(currVelo, 5); //upper bound for velocity
			}
			velo.setMagnitude(currVelo);
			
			//update velocity
			ship.x += velo.x; ship.y += velo.y; ship.rotation = Math2.degreeOf(velo.getAngle());
			
			//reflect when ship is out of stage
			if (ship.x <0 || ship.x > stage.stageWidth) velo.x *= -1;
			if (ship.y <0 || ship.y > stage.stageHeight) velo.y *= -1;
			
			redraw();
			recalculate();
		}
		
		private function redraw():void 
		{
			var cmd:Vector.<int> = new Vector.<int>;
			var coord:Vector.<Number> = new Vector.<Number>;
			
			//redraw curve;
			m1 = new Matrix3d(
				w1.x * w1.x, w1.x, 1, 0,
				w2.x * w2.x, w2.x, 1, 0,
				w3.x * w3.x, w3.x, 1, 0,
				0,0,0,1
			);
			
			m2 = new Matrix3d(
				w1.y, 0, 0, 0,
				w2.y, 0, 0, 0,
				w3.y, 0, 0, 0,
				0,0,0,1
			)
			
			m1.invert();
			m2.append(m1);
			quadratic_equation.define(m2.n11, m2.n21, m2.n31);
			
			minX = Math.min(w1.x, w2.x, w3.x);
			maxX = Math.max(w1.x, w2.x, w3.x);
			
			for (var i:int = minX; i < maxX; i+=2) 
			{
				if (i == minX) cmd.push(1);
				else		cmd.push(2);
				
				coord.push(i, quadratic_equation.fx_of(i));
			}
			
			n1 = new Matrix();
			n1.a = ship.x; 			n1.c = 1;
			n1.b = ship.x + velo.x; n1.d = 1;
			
			n2 = new Matrix();
			n2.a = ship.y; 			n2.c = 0;
			n2.b = ship.y + velo.y; n2.d = 0;
			
			n1.invert();
			n2.concat(n1);
			
			var x:Number = stage.stageWidth		//y = mx + c
			cmd.push(1);	coord.push(0, n2.a * 0 + n2.b);
			cmd.push(2);	coord.push(x, n2.a * x + n2.b);
			
			graphics.clear();
			graphics.lineStyle(1);
			graphics.drawPath(cmd, coord);
		}
		
		private function recalculate():void 
		{
			quadratic_equation.define(m2.n11, m2.n21 - n2.a, m2.n31 - n2.b);
			quadratic_equation.calcRoots();
			
			var roots:Vector.<Number> = quadratic_equation.roots_R;
			
			for (var i:int = 0; i < roots.length; i++) 
			{
				var reposition:Sprite = getChildByName("c" + i) as Sprite
				
				//conditions:
				//real root, value of x within the range
				if (!isNaN(roots[i]) && roots[i] > minX && roots[i] < maxX) {
					reposition.x = roots[i]; 
					reposition.y = n2.a * roots[i] + n2.b;
					
					//discriminating between future and already happened collision point
					var vec:Vector2D = new Vector2D(reposition.x - ship.x, reposition.y - ship.y);
					if (velo.dotProduct(vec) < 0) 	reposition.alpha = 0.4;
					else 							reposition.alpha = 1
				}
				else {
					reposition.x = -100; reposition.y = -100;
				}
			}
		}
	}

}