package  
{
	import flash.display.Sprite;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.ui.Keyboard;
	import fl.controls.Button;
	import EqMath.*;
	/**
	 * ...
	 * @author Shiu
	 */
	[SWF(width = 400, height = 300)]
	public class CollisionTime3 extends Sprite
	{
		private var t:TextField, b1:Button, b2:Button;
		
		//c1 is the gray circle
		//c2 is the red circle
		private var c1:Circle, c2:Circle;
		
		//v1 is the gray circle's velocity
		//v2 is the red circle's velocity
		private var v1:Vector2D, v2:Vector2D, toggle:Boolean = true, usingV1:Boolean = true;
		
		//tri1 will form the arrowhead of the v1
		//tri2 will form the arrowhead of the v2
		private var tri1:Triangle, tri2:Triangle;
		private var container:Sprite;
		private var eq:EqQuadratic;
		
		public function CollisionTime3() 
		{
			t = new TextField; addChild(t);
			t.autoSize = TextFieldAutoSize.LEFT;
			
			b1 = new Button(); addChild(b1); 
			b1.y = 100; b1.label = "Next"
			b1.addEventListener(MouseEvent.MOUSE_DOWN, proceed);
			
			b2 = new Button(); addChild(b2); 
			b2.y = 100+b1.height; b2.label = "Back"
			b2.addEventListener(MouseEvent.MOUSE_DOWN, proceed);
			
			c1 = new Circle(0xAAAAAA); addChild(c1);
			c2 = new Circle(0xFF0000); addChild(c2);
			c1.x = 100;	c2.x = 300;
			c1.y = 250; c2.y = 50;
			c1.addEventListener(MouseEvent.MOUSE_DOWN, move);
			c2.addEventListener(MouseEvent.MOUSE_DOWN, move);
			c1.addEventListener(MouseEvent.MOUSE_UP, move);
			c2.addEventListener(MouseEvent.MOUSE_UP, move);
			
			v1 = new Vector2D(10, 0);	v1.setAngle(Math2.radianOf(-45));
			v2 = new Vector2D(15, 0);	v2.setAngle(Math2.radianOf(130));
			tri1 = new Triangle(); addChild(tri1);
			tri2 = new Triangle(); addChild(tri2);
			container = new Sprite(); addChild(container);
			redraw()
			
			stage.addEventListener(KeyboardEvent.KEY_DOWN, changeVelo)
			
			eq = new EqQuadratic();
		}
		
		private function proceed(e:MouseEvent):void 
		{
			if(e.target.label == "Next"){
				c1.x += v1.x; c1.y += v1.y;
				c2.x += v2.x; c2.y += v2.y;
			}
			else if (e.target.label == "Back"){
				c1.x -= v1.x; c1.y -= v1.y;
				c2.x -= v2.x; c2.y -= v2.y;
			}
			if (toggle) redraw()
			
			getCollisionTime();
		}
		
		private function move(e:MouseEvent):void 
		{
			if (e.type == "mouseDown") e.currentTarget.startDrag();
			else if (e.type == "mouseUp") e.currentTarget.stopDrag();
		}
		
		private function changeVelo(e:KeyboardEvent):void 
		{
			if (e.keyCode == Keyboard.V) {
				if (toggle) hide();
				else		show();
				toggle = !toggle;
			}
			if (e.keyCode == Keyboard.N) usingV1 = !usingV1;
			if (toggle) {
				if (e.keyCode == Keyboard.UP) {
					if (usingV1)	v1.setMagnitude(v1.getMagnitude() + 1);
					else			v2.setMagnitude(v2.getMagnitude() + 1);
				}
				if (e.keyCode == Keyboard.DOWN) {
					if (usingV1)	v1.setMagnitude(v1.getMagnitude() - 1);
					else			v2.setMagnitude(v2.getMagnitude() - 1);
				}
				if (e.keyCode == Keyboard.LEFT) {
					if (usingV1)	v1.setAngle(v1.getAngle() - Math2.radianOf(5));
					else			v2.setAngle(v2.getAngle() - Math2.radianOf(5));
				}
				if (e.keyCode == Keyboard.RIGHT) {
					if (usingV1)	v1.setAngle(v1.getAngle() + Math2.radianOf(5));
					else			v2.setAngle(v2.getAngle() + Math2.radianOf(5));
				}
			}
			redraw();
		}
		
		private function getCollisionTime():void 
		{
			var p:Number = v1.x - v2.x;
			var q:Number = c1.x - c2.x;
			var r:Number = v1.y - v2.y;
			var s:Number = c1.y - c2.y;
			
			var a:Number = p * p + r * r;
			var b:Number = 2 * (p * q + r * s);
			var c:Number = q * q + s * s - (c1.radius + c2.radius) * (c1.radius + c2.radius);
			eq.define(a, b, c);
			eq.calcRoots();
			var roots:Vector.<Number> = eq.roots_R;
			
			//if no real roots are available, then they are on not on collision path
			if (isNaN(roots[0]) && isNaN(roots[1])) {
				t.text = "Particles not on collision path."
			}
			else {
				var time:Number = Math.min(roots[0], roots[1])
				var int_time:int = time * 1000;
				time = int_time / 1000;
				
				t.text = "Frames to impact: "+time.toString() + "\n";
				if(time>1) t.appendText("Particles are closing...")
				else if (time > 0 && time < 1)  t.appendText("Particles WILL collide next frame!")
				else if (time < 0) t.appendText("Collision had already happened.");
			}
		}
		
		private function redraw():void {
			tri1.x = v1.x + c1.x; tri1.y = v1.y + c1.y; tri1.rotation = Math2.degreeOf(v1.getAngle());
			tri2.x = v2.x + c2.x; tri2.y = v2.y + c2.y; tri2.rotation = Math2.degreeOf(v2.getAngle());
			
			with (container) {
				graphics.clear();
				graphics.lineStyle(4);
				graphics.moveTo(c1.x, c1.y)
				graphics.lineTo(tri1.x, tri1.y)
				graphics.moveTo(c2.x, c2.y)
				graphics.lineTo(tri2.x, tri2.y)
			}
		}
		
		private function hide():void {
			tri1.alpha = 0; tri2.alpha = 0; container.alpha = 0;
		}
		private function show():void {
			tri1.alpha = 1; tri2.alpha = 1; container.alpha = 1;
		}
	}

}