package EqMath
{
	/**
	 * This class is used to solve quadratic equations
	 * @author Shiu
	 */
	
	//TODO: Error controls 
	public class EqQuadratic implements IEquation {
		private static var _degree:int = 2;
		
		private var _A:Number, _B:Number, _C:Number;
		private var _root_R:Vector.<Number>;
		private var _root_i:Vector.<String>;
		
		/**
		 * Note the order of degrees written in formula template
		 * f(x) = A (x^2) + B (x) + C
		 */
		public function EqQuadratic() {
			_root_R = new Vector.<Number>;	_root_R.length = 2;
			_root_i = new Vector.<String>;	_root_i.length = 2;
		}
		
		/**
		 * Define the coefficients of quadratic equation
		 * @param	...coeff Coefficients A, B, C
		 */
		public function define (...coeff):void {
			_A = coeff[0]; 	_B = coeff[1]; 	_C = coeff[2];
			
			//reset values
			_root_R[0] = Number.NaN;	_root_i[0] = null; 
			_root_R[1] = Number.NaN;	_root_i[1] = null;
		}
		
		/**
		 * Get f(x) based upon x
		 * @param	x value on x-axis
		 * @return f(x) or value on y-axis
		 */
		public function fx_of(x:Number):Number {
			return _A * x * x +_B * x + _C;
		}
		
		/****************** Begin: Solving for root ******************/
		
		/**Read-only
		 * Discriminant of equation
		 */
		public function get discriminant():Number {
			//B*B-4*A*C
			return _B * _B - 4 * _A * _C;
		}
		
		/**
		 * Performs calculation to obtain roots
		 */
		public function calcRoots():void {
			var disc:Number = this.discriminant
			
			//handle imaginary roots
			if (disc < 0) { 
				disc *= -1;
				var component_real:Number = -_B / (2 * _A);
				var component_imaginary:Number = Math.sqrt(disc) / (2 * _A);
				
				_root_i[0] = (component_real + "+ i" + component_imaginary).toString();
				_root_i[1] = (component_real + "- i" + component_imaginary).toString();
			}
			
			//handle real roots
			else {
				var sqrt:Number = Math.sqrt(disc);
				_root_R[0] = ( -_B + sqrt) / (2 * _A);
				_root_R[1] = ( -_B - sqrt) / (2 * _A);
			}
		}
		/****************** End: Solving for root ******************/
		
		/**
		 * Returns real roots if any exists, else returns NaN
		 */
		public function get roots_R():Vector.<Number> {
			return _root_R;
		}
		
		/**
		 * Returns imaginary roots if any exists, else returns null
		 */
		public function get roots_i():Vector.<String> {
			return _root_i;
		}
		
		/**
		 * Return first degree differentiation
		 * @param	x the x coordinate
		 */
		public function diff1(x:Number):Number {
			/* f(x) = ax ^ 2 + bx + c
			 * f'(x), first degree differentiation
			 * f'(x) = 2ax + b
			 */
			return 2 * _A * x + _B;
		}
		
		/**
		 * Return second degree differentiation
		 * @param	x the x coordinate
		 */
		public function diff2(x:Number):Number {
			/* f(x) = ax ^ 2 + bx + c
			 * f''(X), second degree differentiation
			 * f''(x) = 2a
			 */
			return 2 * _A
		}
	}
}